// Generated by CoffeeScript 1.3.3
(function() {
  var JsonArray, JsonBoolean, JsonErrors, JsonInteger, JsonNumber, JsonObject, JsonProperty, JsonSchema, JsonString, e,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  JsonErrors = (function() {

    function JsonErrors() {
      this.base = [];
      this.attr = {};
    }

    JsonErrors.prototype.add = function(property, error) {
      if (!(property || error)) {
        return;
      }
      if (!(error != null)) {
        error = property;
        property = null;
      }
      if (property) {
        if (error instanceof JsonErrors) {
          return this._mergeErrors(property, error);
        } else {
          return this._addError(property, error);
        }
      } else {
        return this.base.push(error);
      }
    };

    JsonErrors.prototype.addToBase = function(error) {
      return this.add(error);
    };

    JsonErrors.prototype.on = function(property) {
      if (property) {
        return this.attr[property];
      } else {
        return this.base;
      }
    };

    JsonErrors.prototype.onBase = function() {
      return this.on();
    };

    JsonErrors.prototype.all = function() {
      var base, err, key;
      base = this.base.length ? [["", this.base]] : [];
      return base.concat((function() {
        var _ref, _results;
        _ref = this.attr;
        _results = [];
        for (key in _ref) {
          err = _ref[key];
          _results.push([key, err]);
        }
        return _results;
      }).call(this));
    };

    JsonErrors.prototype.isEmpty = function() {
      return this.base.length === 0 && Object.keys(this.attr).length === 0;
    };

    JsonErrors.prototype._addError = function(property, error) {
      var _base;
      (_base = this.attr)[property] || (_base[property] = []);
      if (error.length) {
        return this.attr[property] = this.attr[property].concat(error);
      } else {
        return this.attr[property].push(error);
      }
    };

    JsonErrors.prototype._mergeErrors = function(property, errors) {
      var err, newProp, prop, _i, _len, _ref, _ref1, _results;
      if (errors.isEmpty()) {
        return;
      }
      _ref = errors.all();
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        _ref1 = _ref[_i], prop = _ref1[0], err = _ref1[1];
        newProp = prop ? "" + property + "." + prop : property;
        _results.push(this.add(newProp, err));
      }
      return _results;
    };

    return JsonErrors;

  })();

  JsonProperty = (function() {

    function JsonProperty(attr) {
      this.attr = attr;
    }

    JsonProperty.prototype.cast = function(val) {
      return val;
    };

    JsonProperty.prototype.errors = function(val) {
      var errors;
      errors = new JsonErrors;
      if (!this.validate("required", function(r) {
        return !(r && typeof val === "undefined");
      })) {
        errors.add("required");
      }
      return errors;
    };

    JsonProperty.prototype.process = function(val) {
      var errors;
      val = val != null ? this.cast(val) : this.attr["default"];
      errors = this.errors(val);
      return {
        valid: errors.isEmpty(),
        doc: val,
        errors: errors
      };
    };

    JsonProperty.prototype.validate = function(attr, fn) {
      if ((attr in this.attr) && this.attr[attr] !== null) {
        return fn.call(this, this.attr[attr]);
      } else {
        return true;
      }
    };

    return JsonProperty;

  })();

  JsonString = (function(_super) {

    __extends(JsonString, _super);

    function JsonString() {
      return JsonString.__super__.constructor.apply(this, arguments);
    }

    JsonString.prototype.cast = function(val) {
      switch (this.attr.format) {
        case "date":
        case "date-time":
          return new Date(val);
        default:
          return val.toString();
      }
    };

    JsonString.prototype.errors = function(val) {
      var errors;
      errors = JsonString.__super__.errors.call(this, val);
      if (val != null) {
        if (!this.validate("minLength", function(len) {
          return val.length >= len;
        })) {
          errors.add("minLength");
        }
        if (!this.validate("maxLength", function(len) {
          return val.length <= len;
        })) {
          errors.add("maxLength");
        }
        if (!this.validate("pattern", function(pat) {
          return new RegExp(pat).test(val);
        })) {
          errors.add("pattern");
        }
        if (!this.validate("enum", function(opts) {
          return __indexOf.call(opts, val) >= 0;
        })) {
          errors.add("enum");
        }
        if (!this.validate("format", function(format) {
          return this.validFormat(format, val);
        })) {
          errors.add("format");
        }
      }
      return errors;
    };

    JsonString.prototype.validFormat = function(format, val) {
      switch (this.attr.format) {
        case "date":
          return /^\d\d\d\d-\d\d-\d\d$/.test(val);
        case "date-time":
          return /^\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\dZ$/.test(val);
        default:
          return true;
      }
    };

    return JsonString;

  })(JsonProperty);

  JsonNumber = (function(_super) {

    __extends(JsonNumber, _super);

    function JsonNumber() {
      return JsonNumber.__super__.constructor.apply(this, arguments);
    }

    JsonNumber.prototype.cast = function(val) {
      val = parseFloat(val);
      if (isNaN(val)) {
        return null;
      } else {
        return val;
      }
    };

    JsonNumber.prototype.errors = function(val) {
      var errors;
      errors = JsonNumber.__super__.errors.call(this, val);
      if (val != null) {
        if (!this.validate("minimum", function(min) {
          if (this.attr.excludeMinimum) {
            return val > min;
          } else {
            return val >= min;
          }
        })) {
          errors.add("minimum");
        }
        if (!this.validate("maximum", function(max) {
          if (this.attr.excludeMaximum) {
            return val < max;
          } else {
            return val <= max;
          }
        })) {
          errors.add("maximum");
        }
        if (!this.validate("divisibleBy", function(div) {
          return val % div === 0;
        })) {
          errors.add("divisibleBy");
        }
      }
      return errors;
    };

    return JsonNumber;

  })(JsonProperty);

  JsonInteger = (function(_super) {

    __extends(JsonInteger, _super);

    function JsonInteger() {
      return JsonInteger.__super__.constructor.apply(this, arguments);
    }

    JsonInteger.prototype.cast = function(val) {
      val = parseInt(val, 10);
      if (isNaN(val)) {
        return null;
      } else {
        return val;
      }
    };

    return JsonInteger;

  })(JsonNumber);

  JsonBoolean = (function(_super) {

    __extends(JsonBoolean, _super);

    function JsonBoolean() {
      return JsonBoolean.__super__.constructor.apply(this, arguments);
    }

    JsonBoolean.prototype.cast = function(val) {
      if (val === void 0) {
        return;
      }
      if (val === "false" || val === "null" || val === "0") {
        return false;
      } else if (val) {
        return true;
      } else {
        return false;
      }
    };

    return JsonBoolean;

  })(JsonProperty);

  JsonArray = (function(_super) {

    __extends(JsonArray, _super);

    function JsonArray(attr) {
      var ref;
      this.attr = attr;
      if (this.attr.items) {
        ref = this.attr.items["$ref"];
        this.itemSchema = ref ? JsonSchema.resolver(this.attr.items["$ref"], this) : JsonProperty["for"](this.attr.items);
      }
    }

    JsonArray.prototype.cast = function(val) {
      var cast, item, _i, _len, _results,
        _this = this;
      cast = this.itemSchema ? function(v) {
        return _this.itemSchema.cast(v);
      } : function(v) {
        return v;
      };
      _results = [];
      for (_i = 0, _len = val.length; _i < _len; _i++) {
        item = val[_i];
        _results.push(cast(item));
      }
      return _results;
    };

    JsonArray.prototype.errors = function(val) {
      var errors, i, item, _i, _len;
      errors = JsonArray.__super__.errors.call(this, val);
      if (val != null) {
        if (!this.validate("minItems", function(min) {
          return val.length >= min;
        })) {
          errors.add("minItems");
        }
        if (!this.validate("maxItems", function(max) {
          return val.length <= max;
        })) {
          errors.add("maxItems");
        }
        if (this.itemSchema) {
          for (i = _i = 0, _len = val.length; _i < _len; i = ++_i) {
            item = val[i];
            errors.add("" + i, this.itemSchema.errors(item));
          }
        }
      }
      return errors;
    };

    return JsonArray;

  })(JsonProperty);

  JsonObject = (function(_super) {

    __extends(JsonObject, _super);

    function JsonObject(attr) {
      this.attr = attr;
      this.properties = attr.properties;
      if (this.properties["$ref"]) {
        this.ref = JsonSchema.resolver(this.properties["$ref"].replace(/#.+$/, ''), this);
      }
    }

    JsonObject.prototype.cast = function(val) {
      var attrs, key, obj, _ref;
      if (this.ref) {
        return this.ref.cast(val);
      }
      obj = {};
      _ref = this.properties;
      for (key in _ref) {
        attrs = _ref[key];
        obj[key] = val && (key in val) ? JsonProperty["for"](attrs).cast(val[key]) : attrs["default"];
      }
      return obj;
    };

    JsonObject.prototype.process = function(val) {
      if (this.ref) {
        return this.ref.process(val);
      } else {
        return JsonObject.__super__.process.call(this, val);
      }
    };

    JsonObject.prototype.errors = function(val) {
      var attrs, err, errors, key, _ref;
      if (val == null) {
        return JsonObject.__super__.errors.call(this, val);
      }
      if (this.ref) {
        return this.ref.errors(val);
      }
      errors = JsonObject.__super__.errors.call(this, val);
      _ref = this.properties;
      for (key in _ref) {
        attrs = _ref[key];
        err = JsonProperty["for"](attrs).errors(val && val[key]);
        errors.add(key, err);
      }
      return errors;
    };

    return JsonObject;

  })(JsonProperty);

  JsonProperty["for"] = function(attr) {
    var klass, type;
    type = attr.type || "any";
    klass = {
      "any": JsonProperty,
      "string": JsonString,
      "number": JsonNumber,
      "integer": JsonInteger,
      "boolean": JsonBoolean,
      "array": JsonArray,
      "object": JsonObject
    }[type];
    if (!klass) {
      throw "Bad Schema - Unknown property type " + type;
    }
    return new klass(attr);
  };

  JsonSchema = (function(_super) {

    __extends(JsonSchema, _super);

    function JsonSchema(attr) {
      if (attr.type !== "object") {
        throw "The main schema must be of type \"object\"";
      }
      JsonSchema.__super__.constructor.call(this, attr);
    }

    return JsonSchema;

  })(JsonObject);

  JsonSchema.resolver = function(url) {
    if (!JsonSchema.resolver) {
      throw "No resolver defined for references";
    }
  };

  e = ((typeof exports !== "undefined" && exports !== null) && exports) || ((typeof window !== "undefined" && window !== null) && window);

  e.JsonSchema = JsonSchema;

  e.JsonErrors = JsonErrors;

}).call(this);
